@@ -93,7 +93,7 @@ pub struct IPv6AddrList<const MAX: usize> {
 }
 
 impl<const MAX: usize> IPv6AddrList<MAX> {
-    #[deprecated(note = "IPv6AddrList now derefs into a slice")]
+    #[deprecated(note = "&IPv6AddrList now implements IntoIterator")]
     pub fn addresses(&self) -> &[IPv6Addr] {
         self
     }
@@ -109,6 +109,16 @@ impl<const MAX: usize> core::ops::Deref for IPv6AddrList<MAX> {
     }
 }
 
+impl<'a, const MAX: usize> core::iter::IntoIterator for &'a IPv6AddrList<MAX> {
+    type Item = &'a IPv6Addr;
+
+    type IntoIter = core::slice::Iter<'a, IPv6Addr>;
+
+    fn into_iter(self) -> Self::IntoIter {
+        self[..].iter()
+    }
+}
+
 #[repr(transparent)] // which allows the IPv6AddrList addresss to be passed to gnrc_netif_ipv6_addrs_get
 pub struct IPv6Addr {
     inner: ipv6_addr_t,
