@@ -1,15 +1,14 @@
 //! A safe-to-use interface for [RIOT messages](https://doc.riot-os.org/group__core__msg.html)
 //!
-//! This module's main contribution is [MessageSemantics], a chaining family of ZST types that
-//! represent which types a thread expects to come in on which message numbers. From that, sendable
-//! [MessageAddressTicket]s can be crated that indicate to message producers that indeed the
-//! indicated thread is ready to accept messages of some type on that message type.
+//! This module's main contribution is [MessageSemantics], a chaining family of ZSTs that
+//! represent which types a thread expects to come in on which message numbers. From that, pairs of
+//! [SendPort]s (through which other threads send messages) and [ReceivePort]s (using which
+//! received messages are decoded) are split off.
 //!
 //! For safety, the module relies on other components not tossing around messages indiscriminately.
-//! Rust components can take a [MessageAddressTicket] and send through that (where objects'
-//! lifetimes guarantee that the receiving task is still prepared to receive the message). For C
-//! components, safe wrappers (TBD: will) require appropriate tickets and from there on rely on the
-//! C side to only send what is described in the API.
+//! In Rust, senders are told through the SendPort how the recipient will transmute the data back.
+//! For C components, safe wrappers (TBD: will) require and consume appropriate tickets and from
+//! there on rely on the C side to only send what is described in the API.
 //!
 //! ## Example
 //!
@@ -21,6 +20,22 @@
 //! This module is still WIP and not subject to the semver-ish conduct upheld even in pre-1.0
 //! versions of riot-wrappers. The module is hidden behind the `with_msg_v2` feature to make that
 //! clear.
+//!
+//! ## Status vs. Road map
+//!
+//! Conceptually, this module is built for threads that can shut down again, or for receiving
+//! messages only in defined times (eg. when ). All information is preserved to allow ports split
+//! off from a thread's message number pool to be recombined, pending messages to be drained and
+//! the thread set up to receive completely different messages on the same message type numbers.
+//!
+//! Practically, this is not implemented, because threads are not commonly used in a
+//! run-and-shut-down pattern in RIOT. Some documentation refers to the process of recombination
+//! already.
+//!
+//! It might still be tricky to actually perform that recombination safely, as there can be queued
+//! up messages. One pattern that is anticipated to work here is to define a single channel on
+//! which a SendPort can be returned through the message queue; thus, by the time it gets returned,
+//! the receiver can be sure that nothing else is left in the queue.
 
 use core::marker::PhantomData;
 use core::mem::{ManuallyDrop, MaybeUninit};
@@ -29,50 +44,52 @@ use crate::thread;
 
 /// Thread-bound information carrier that indicates that a given type number was reserved for this
 /// given content type.
-pub struct MessagePort<TYPE: Send, const TYPENO: u16> {
+pub struct ReceivePort<TYPE: Send, const TYPENO: u16> {
     // Can only be constructed by split_off()
     _private: (),
     // Regular PhantomData for information -- information is needd by several impl fns.
     _types: PhantomData<TYPE>,
-    // Ensure MessagePort is not Send or Sync -- it is specific to the one thread it's born in
+    // Ensure ReceivePort is not Send or Sync -- it is specific to the one thread it's born in
     // because it contains a statement about the current thread.
     //
-    // (The alternative would be to brand the MessagePort to the thread, but that only works well
+    // (The alternative would be to brand the ReceivePort to the thread, but that only works well
     // once there is a per-thread brand, eg. created along the mechansim described in the
     // NoConfiguredMessages::new TBD).
     _not_send: PhantomData<*const ()>,
 }
 
-impl<TYPE: Send, const TYPENO: u16> MessagePort<TYPE, TYPENO> {
-    // TBD: Can't we get the promise to not quit the thread any more easily than passing lifetimes
-    // with each and every ticket? Probably not. But can we get the promise in any more easily than
-    // the thread unsafely stating it by upgrading its message port lifetime?
-    //
-    // (Plus it'll be extraordinarily tricky to do anything non-static here -- you'd need to return
-    // the handed out ticket, then work the message queue to emptiness (or, if it never gets empty,
-    // past the point when the ticket was returned) and only then can the port expire ... and all
-    // of that needs to be encoded in the type system.
-    pub fn ticket(&self) -> MessageAddressTicket<'_, TYPE, TYPENO> {
-        MessageAddressTicket {
-            destination: thread::get_pid(),
-            _phantom: PhantomData,
-        }
-    }
-}
+// FIXME: Maybe implement Deref to SendPort so the port can be used to send to itself?
 
-pub struct MessageAddressTicket<'a, TYPE: Send, const TYPENO: u16> {
+/// Object through which messages of a precise type can be sent to a precise thread.
+///
+/// Unlike the ReceivePort, the SendPort is Send and Sync by addign the runtime information of the
+/// destination Kernel PID to it. That process / thread is guaranteed to be live (might have
+/// crashed to a non-unwinding panic but not been reused) by the construction of SendPort: A
+/// SendPort can only be created when the indicated thread gives the appropriate guarantees.
+///
+/// It is owned, but can be used through shared references (which are Send as well); ownership
+/// matters if one ever wants to stop accepting a certain type of message again.
+///
+/// If it is desired that multiple callers send on a single typeno (where the callers can not just
+/// share a shared reference), it would be possible to create a version of the SendPort
+/// that counts its clones at runtime and can only be returned when all of them are recombined, or
+/// just to create a version that can be cloned at will but never recombined any more. (One way to
+/// do the latter would be to add a const boolean type parameter "CLONED"; a `.clonable(self) ->
+/// Self` would switch that from false to true, and then copy and clone would be implemented for
+/// the result, whereas recombination would only be implemented for the CLONED = false version).
+pub struct SendPort<TYPE: Send, const TYPENO: u16> {
     destination: thread::KernelPID,
-    _phantom: PhantomData<&'a TYPE>,
+    _phantom: PhantomData<TYPE>,
 }
 
-impl<'a, TYPE: Send, const TYPENO: u16> MessageAddressTicket<'a, TYPE, TYPENO> {
+impl<TYPE: Send, const TYPENO: u16> SendPort<TYPE, TYPENO> {
     /// Send a message to a given ticket.
     ///
     /// On success, the data is received by (or enqueued in, if a queue is set up) the thread
     /// indicated in the ticket. Otherwise, the data is returned.
     ///
     /// Note that while the underlying `msg_try_send` function knows two error cases (thread is not
-    /// ready to receive, and invalid PID), the presence of a MessageAddressTicket implies that the
+    /// ready to receive, and invalid PID), the presence of a SendPort implies that the
     /// thread promised to still be around (it may have crashed, but it can't have exited), so that
     /// error can not happen here. (If it still does due to errors in unsafe code, trips up a debug
     /// assert and else is handled like the other failure to send).
@@ -91,7 +108,7 @@ impl<'a, TYPE: Send, const TYPENO: u16> MessageAddressTicket<'a, TYPE, TYPENO> {
         // invalid one.
         debug_assert!(
             result >= 0,
-            "Target PID vanished even though a MessageAddressTicket was still around"
+            "Target PID vanished even though a SendPort was still around"
         );
         match result {
             1 => Ok(()),
@@ -104,15 +121,34 @@ impl<'a, TYPE: Send, const TYPENO: u16> MessageAddressTicket<'a, TYPE, TYPENO> {
             }
         }
     }
+
+    /// Access the port's destination
+    ///
+    /// This is particularly useful when messages are not sent directly through [`try_send()`], but
+    /// the port is stored (or dropped) after having been typechecked to match the described API of
+    /// a C function that will send messages -- and after having extracted the destination for
+    /// these messages with this function.
+    pub fn destination(&self) -> thread::KernelPID {
+        self.destination
+    }
 }
 
-impl<'a, TYPE: Send, const TYPENO: u16> core::fmt::Debug
-    for MessageAddressTicket<'a, TYPE, TYPENO>
-{
+impl<TYPE: Send, const TYPENO: u16> core::fmt::Debug for ReceivePort<TYPE, TYPENO> {
     fn fmt(&self, f: &mut core::fmt::Formatter<'_>) -> Result<(), core::fmt::Error> {
         write!(
             f,
-            "MessageAddressTicket<{}, {}> {{ destination: {:?} }}",
+            "ReceivePort<{}, {}> {{ }}",
+            core::any::type_name::<TYPE>(),
+            TYPENO,
+        )
+    }
+}
+
+impl<TYPE: Send, const TYPENO: u16> core::fmt::Debug for SendPort<TYPE, TYPENO> {
+    fn fmt(&self, f: &mut core::fmt::Formatter<'_>) -> Result<(), core::fmt::Error> {
+        write!(
+            f,
+            "SendPort<{}, {}> {{ destination: {:?} }}",
             core::any::type_name::<TYPE>(),
             TYPENO,
             self.destination
@@ -120,19 +156,23 @@ impl<'a, TYPE: Send, const TYPENO: u16> core::fmt::Debug
     }
 }
 
+/// Trait for types that indicate the current thread's readiness to receive some set of messages
+///
+/// In a sense, a MessageSemantics is factory for mutually nonconflicting [ReceivePort]s, and a
+/// tracker of what was alerady issued.
 // TBD: seal? can still unseal later.
 pub trait MessageSemantics: Sized {
     // TBD: Would be great to be const
     fn typeno_is_known(&self, typeno: u16) -> bool;
 
     /// Reduce the type into a new MessageSemantics that knows about one more typeno, and a
-    /// MessagePort that can be used to create a [MessageAddressTicket] or to process incoming
+    /// ReceivePort that can be used to create a [SendPort] or to process incoming
     /// messages.
     ///
     /// ```
-    /// # use riot_wrappers::msg::v2::MessagePort;
-    /// type NumberReceived = MessagePort<u32, 1>;
-    /// type BoolReceibed = MessagePort<bool, 2>;
+    /// # use riot_wrappers::msg::v2::ReceivePort;
+    /// type NumberReceived = ReceivePort<u32, 1>;
+    /// type BoolReceibed = ReceivePort<bool, 2>;
     /// let (message_semantics, mnum): (_, NumberReceived) = message_semantics.split_off();
     /// let (message_semantics, mbool): (_, BoolReceived) = message_semantics.split_off();
     /// ```
@@ -145,11 +185,12 @@ pub trait MessageSemantics: Sized {
     /// The conditions for these panics should be evaluatable at build time (i.e. not be part of
     /// optimized code); over time these will hopfully become static assertion errors.
     // No override should be necessary for this, not even for internal impls (see sealing above)
-    fn split_off<NEW_TYPE: Send, const NEW_TYPENO: u16>(
+    fn split_off<NewType: Send, const NEW_TYPENO: u16>(
         self,
     ) -> (
-        Processing<Self, NEW_TYPE, NEW_TYPENO>,
-        MessagePort<NEW_TYPE, NEW_TYPENO>,
+        Processing<Self, NewType, NEW_TYPENO>,
+        ReceivePort<NewType, NEW_TYPENO>,
+        SendPort<NewType, NEW_TYPENO>,
     ) {
         // Should ideally be a static assert. Checks probably happen at build time anyway due to
         // const propagation, but the panic only triggers at runtime :-(
@@ -160,7 +201,7 @@ pub trait MessageSemantics: Sized {
 
         // Similarly static -- better err out early
         assert!(
-            core::mem::size_of::<NEW_TYPE>()
+            core::mem::size_of::<NewType>()
                 <= core::mem::size_of::<riot_sys::msg_t__bindgen_ty_1>(),
             "Type is too large to be transported in a message"
         );
@@ -168,7 +209,7 @@ pub trait MessageSemantics: Sized {
         // ... and the alignment must suffice because the data is moved in and outthrough a &mut
         // SomethingTransparent<T>
         assert!(
-            core::mem::align_of::<NEW_TYPE>()
+            core::mem::align_of::<NewType>()
                 <= core::mem::align_of::<riot_sys::msg_t__bindgen_ty_1>(),
             "Type has stricter alignment requirements than the message content"
         );
@@ -178,16 +219,20 @@ pub trait MessageSemantics: Sized {
                 tail: self,
                 _type: PhantomData,
             },
-            MessagePort {
+            ReceivePort {
                 _private: (),
                 _types: PhantomData,
                 _not_send: PhantomData,
             },
+            SendPort {
+                destination: thread::get_pid(),
+                _phantom: PhantomData,
+            },
         )
     }
 
     // No override should be necessary for this, not even for internal impls (see sealing above)
-    fn receive(&self) -> ReceivedMessage<Self> {
+    fn receive(&self) -> ReceivedMessage<'_, Self> {
         let mut msg = MaybeUninit::uninit();
         unsafe { riot_sys::msg_receive(msg.as_mut_ptr()) };
         let msg = unsafe { msg.assume_init() };
@@ -211,22 +256,44 @@ pub trait MessageSemantics: Sized {
     ///
     /// This is unsafe for the same reasons you can't call Drop::drop(&mut T) (the compiler forbids
     /// it).
-    unsafe fn drop(message: &mut ReceivedMessage<Self>);
+    unsafe fn drop(message: &mut ReceivedMessage<'_, Self>);
 }
 
 pub struct NoConfiguredMessages;
 
+/// The MessageSemantics of a thread that has made no previous commitment to receive any
+/// messages.
 impl NoConfiguredMessages {
-    /// The MessageSemantics of a thread that has made no previous commitment to receive any
-    /// messages.
+    /// Create a new MessageSemantics object to split into [ReceivePort]s.
     ///
-    /// It must only be called once in such a thread.
+    /// **Conditions**, violating which is a safety violation:
+    ///
+    /// * The thread must currently not allow sending any messages to it, or even created an
+    ///   otherwise unused NoConfiguredMessages
+    ///
+    /// * The thread must not terminate.
     ///
     /// TBD: Add a version of the thread spawner that comes with all kinds of once-per-thread
     /// gadgets.
     pub unsafe fn new() -> Self {
         Self
     }
+
+    /// Create a new MessageSemantics object to split into [ReceivePort]s in a scope.
+    ///
+    /// This is somewhat safer to use than [new()] because by taking the NoConfiguredMessages
+    /// object back (which currently can only be done by not splitting off anything, and later by
+    /// returning everything that was split off); an easy way to do that is to just never return.
+    ///
+    /// **Conditions**, violating which is a safety violation:
+    ///
+    /// * The thread must currently not allow sending any messages to it, or even created an
+    ///   otherwise unused NoConfiguredMessages
+    pub unsafe fn new_scoped(f: impl FnOnce(Self) -> Self) {
+        f(Self);
+        // FIXME: ensure that the queue is flushed (eg. by sending a terminal message, or by
+        // asserting that the queue is empty)
+    }
 }
 
 impl MessageSemantics for NoConfiguredMessages {
@@ -242,7 +309,7 @@ impl MessageSemantics for NoConfiguredMessages {
     /// happen to send messages of a number this threads expects (and handles as something
     /// different) as well. If it is still undesired, you can [core::mem::forget()] the message
     /// after having decoded all desired types.
-    unsafe fn drop(_message: &mut ReceivedMessage<Self>) {
+    unsafe fn drop(_message: &mut ReceivedMessage<'_, Self>) {
         panic!("Unexpected message received");
     }
 }
@@ -250,8 +317,8 @@ impl MessageSemantics for NoConfiguredMessages {
 pub struct Processing<TAIL: MessageSemantics, TYPE, const TYPENO: u16> {
     tail: TAIL,
     // Carried around solely to be able to drop messages that did not get decoded. (Otherwise we'd
-    // take solace in the fact that the MessagePort knows how to handle it, and there'll only be
-    // one MessagePort of a given type in a thread, and either that one takes the message or nobody
+    // take solace in the fact that the ReceivePort knows how to handle it, and there'll only be
+    // one ReceivePort of a given type in a thread, and either that one takes the message or nobody
     // does and it'd get dropped).
     _type: PhantomData<TYPE>,
 }
@@ -267,7 +334,7 @@ impl<TAIL: MessageSemantics, TYPE, const TYPENO: u16> MessageSemantics
         }
     }
 
-    unsafe fn drop(message: &mut ReceivedMessage<Self>) {
+    unsafe fn drop(message: &mut ReceivedMessage<'_, Self>) {
         if message.msg.type_ == TYPENO {
             let t: TYPE = message.extract();
             drop(t);
@@ -279,13 +346,21 @@ impl<TAIL: MessageSemantics, TYPE, const TYPENO: u16> MessageSemantics
 
 pub use crate::msg::MsgSender as Sender;
 
+/// A message that was received while S was the current thread's semantics.
+///
+/// By including a lifetime argument this ensures that messages are decoded (or dropped) before
+/// the thread's message semantics change.
+///
+/// It may help to think of this as still holding a pointer to 's, just that it's a zero-sized
+/// phantom pointer (no need to lug around a word-sized pointer to a ZST object that's just there
+/// for typestate).
 #[repr(transparent)]
-pub struct ReceivedMessage<S: MessageSemantics> {
+pub struct ReceivedMessage<'a, S: MessageSemantics> {
     msg: riot_sys::msg_t,
-    _phantom: PhantomData<S>,
+    _phantom: PhantomData<&'a S>,
 }
 
-impl<S: MessageSemantics> core::fmt::Debug for ReceivedMessage<S> {
+impl<'a, S: MessageSemantics> core::fmt::Debug for ReceivedMessage<'a, S> {
     fn fmt(&self, f: &mut core::fmt::Formatter<'_>) -> Result<(), core::fmt::Error> {
         write!(
             f,
@@ -296,14 +371,14 @@ impl<S: MessageSemantics> core::fmt::Debug for ReceivedMessage<S> {
     }
 }
 
-impl<S: MessageSemantics> Drop for ReceivedMessage<S> {
+impl<'a, S: MessageSemantics> Drop for ReceivedMessage<'a, S> {
     #[inline]
     fn drop(&mut self) {
         unsafe { S::drop(self) };
     }
 }
 
-impl<S: MessageSemantics> ReceivedMessage<S> {
+impl<'a, S: MessageSemantics> ReceivedMessage<'a, S> {
     fn sender(&self) -> Sender {
         Sender::from_pid(self.msg.sender_pid)
     }
@@ -311,7 +386,7 @@ impl<S: MessageSemantics> ReceivedMessage<S> {
     #[inline]
     /// Move the T out of self, leaving the msg partially uninitialized
     ///
-    /// This can only be used on a type T that a MessagePort was created for.
+    /// This can only be used on a type T that a ReceivePort was created for.
     unsafe fn extract<T>(&mut self) -> T {
         // This'd be easier if we'd constrain transmuted to Clone...
         let mut transmuted = MaybeUninit::uninit();
@@ -325,7 +400,7 @@ impl<S: MessageSemantics> ReceivedMessage<S> {
 
     pub fn decode<R, F: FnOnce(Sender, TYPE) -> R, TYPE: Send, const TYPENO: u16>(
         mut self,
-        _port: &MessagePort<TYPE, TYPENO>,
+        _port: &'a ReceivePort<TYPE, TYPENO>,
         f: F,
     ) -> Result<R, ReceivedMessage<S>> {
         // Not actually using the port argument, it's just the ZST on whose presence the type
